@@ -1,6 +1,6 @@
 # -*- test-case-name: twisted.test.test_amp -*-
 # Copyright (c) 2005 Divmod, Inc.
-# Copyright (c) 2007-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2007-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -167,6 +167,8 @@ from struct import pack
 
 from zope.interface import Interface, implements
 
+from twisted.python.compat import set
+from twisted.python.util import unsignedID
 from twisted.python.reflect import accumulateClassDict
 from twisted.python.failure import Failure
 from twisted.python import log, filepath
@@ -176,7 +178,17 @@ from twisted.internet.error import PeerVerifyError, ConnectionLost
 from twisted.internet.error import ConnectionClosed
 from twisted.internet.defer import Deferred, maybeDeferred, fail
 from twisted.protocols.basic import Int16StringReceiver, StatefulStringProtocol
-from twisted.internet.ssl import CertificateOptions, Certificate, DN, KeyPair
+
+try:
+    from twisted.internet import ssl
+except ImportError:
+    ssl = None
+
+if ssl and not ssl.supported:
+    ssl = None
+
+if ssl is not None:
+    from twisted.internet.ssl import CertificateOptions, Certificate, DN, KeyPair
 
 ASK = '_ask'
 ANSWER = '_answer'
@@ -191,6 +203,71 @@ MAX_KEY_LENGTH = 0xff
 MAX_VALUE_LENGTH = 0xffff
 
 
+class IArgumentType(Interface):
+    """
+    An L{IArgumentType} can serialize a Python object into an AMP box and
+    deserialize information from an AMP box back into a Python object.
+
+    @since: 9.0
+    """
+    def fromBox(name, strings, objects, proto):
+        """
+        Given an argument name and an AMP box containing serialized values,
+        extract one or more Python objects and add them to the C{objects}
+        dictionary.
+
+        @param name: The name associated with this argument.  Most commonly,
+            this is the key which can be used to find a serialized value in
+            C{strings} and which should be used as the key in C{objects} to
+            associate with a structured Python object.
+        @type name: C{str}
+
+        @param strings: The AMP box from which to extract one or more
+            values.
+        @type strings: C{dict}
+
+        @param objects: The output dictionary to populate with the value for
+            this argument.
+        @type objects: C{dict}
+
+        @param proto: The protocol instance which received the AMP box being
+            interpreted.  Most likely this is an instance of L{AMP}, but
+            this is not guaranteed.
+
+        @return: C{None}
+        """
+
+
+    def toBox(name, strings, objects, proto):
+        """
+        Given an argument name and a dictionary containing structured Python
+        objects, serialize values into one or more strings and add them to
+        the C{strings} dictionary.
+
+        @param name: The name associated with this argument.  Most commonly,
+            this is the key which can be used to find an object in
+            C{objects} and which should be used as the key in C{strings} to
+            associate with a C{str} giving the serialized form of that
+            object.
+        @type name: C{str}
+
+        @param strings: The AMP box into which to insert one or more
+            strings.
+        @type strings: C{dict}
+
+        @param objects: The input dictionary from which to extract Python
+            objects to serialize.
+        @type objects: C{dict}
+
+        @param proto: The protocol instance which will send the AMP box once
+            it is fully populated.  Most likely this is an instance of
+            L{AMP}, but this is not guaranteed.
+
+        @return: C{None}
+        """
+
+
+
 class IBoxSender(Interface):
     """
     A transport which can send L{AmpBox} objects.
@@ -678,7 +755,7 @@ class BoxDispatcher:
         this command, or None, as specified by requiresAnswer.
         """
         box = Box(kw)
-        return self._sendBoxCommand(command, box)
+        return self._sendBoxCommand(command, box, requiresAnswer)
 
 
     def callRemote(self, commandType, *a, **kw):
@@ -718,7 +795,10 @@ class BoxDispatcher:
         # In other words, the fact that commandType is instantiated here is an
         # implementation detail.  Don't rely on it.
 
-        co = commandType(*a, **kw)
+        try:
+            co = commandType(*a, **kw)
+        except:
+            return fail()
         return co._doCommand(self)
 
 
@@ -1042,7 +1122,14 @@ class Argument:
     """
     Base-class of all objects that take values from Amp packets and convert
     them into objects for Python functions.
+
+    This implementation of L{IArgumentType} provides several higher-level
+    hooks for subclasses to override.  See L{toString} and L{fromString}
+    which will be used to define the behavior of L{IArgumentType.toBox} and
+    L{IArgumentType.fromBox}, respectively.
     """
+    implements(IArgumentType)
+
     optional = False
 
 
@@ -1283,14 +1370,18 @@ class AmpList(Argument):
         AmpList([('a', Integer()),
                  ('b', Unicode())])
     """
-    def __init__(self, subargs):
+    def __init__(self, subargs, optional=False):
         """
         Create an AmpList.
 
         @param subargs: a list of 2-tuples of ('name', argument) describing the
         schema of the dictionaries in the sequence of amp boxes.
+        
+        @param optional: a boolean indicating whether this argument can be
+        omitted in the protocol.
         """
         self.subargs = subargs
+        Argument.__init__(self, optional)
 
 
     def fromStringProto(self, inString, proto):
@@ -1417,8 +1508,11 @@ class Command:
 
         @return: an L{AmpBox}.
         """
-        return _objectsToStrings(objects, cls.response, cls.responseType(),
-                                 proto)
+        try:
+            responseType = cls.responseType()
+        except:
+            return fail()
+        return _objectsToStrings(objects, cls.response, responseType, proto)
     makeResponse = classmethod(makeResponse)
 
 
@@ -1435,6 +1529,14 @@ class Command:
 
         @return: An instance of this L{Command}'s C{commandType}.
         """
+        allowedNames = set()
+        for (argName, ignored) in cls.arguments:
+            allowedNames.add(_wireNameToPythonIdentifier(argName))
+
+        for intendedArg in objects:
+            if intendedArg not in allowedNames:
+                raise InvalidSignature(
+                    "%s is not a valid argument" % (intendedArg,))
         return _objectsToStrings(objects, cls.arguments, cls.commandType(),
                                  proto)
     makeArguments = classmethod(makeArguments)
@@ -1598,6 +1700,12 @@ class _TLSBox(AmpBox):
     """
     __slots__ = []
 
+    def __init__(self):
+        if ssl is None:
+            raise RemoteAmpError("TLS_ERROR", "TLS not available")
+        AmpBox.__init__(self)
+
+
     def _keyprop(k, default):
         return property(lambda self: self.get(k, default))
 
@@ -1669,6 +1777,8 @@ class StartTLS(Command):
         @param tls_verifyAuthorities: a list of Certificate objects which
         represent root certificates to verify our peer with.
         """
+        if ssl is None:
+            raise RuntimeError("TLS not available.")
         self.certificate = kw.pop('tls_localCertificate', _NoCertificate(True))
         self.authorities = kw.pop('tls_verifyAuthorities', None)
         Command.__init__(self, **kw)
@@ -2082,9 +2192,12 @@ class AMP(BinaryBoxProtocol, BoxDispatcher,
         A verbose string representation which gives us information about this
         AMP connection.
         """
-        return '<%s %s at 0x%x>' % (
-            self.__class__.__name__,
-            self.innerProtocol, id(self))
+        if self.innerProtocol is not None:
+            innerRepr = ' inner %r' % (self.innerProtocol,)
+        else:
+            innerRepr = ''
+        return '<%s%s at 0x%x>' % (
+            self.__class__.__name__, innerRepr, unsignedID(self))
 
 
     def makeConnection(self, transport):
