@@ -1,31 +1,55 @@
 # -*- test-case-name: twisted.trial.test.test_runner -*-
-
-#
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 A miscellany of code used to run Trial tests.
 
-Maintainer: Jonathan Lange <jml@twistedmatrix.com>
+Maintainer: Jonathan Lange
 """
 
 
-from __future__ import generators
-import pdb, shutil, sets
+import pdb
 import os, types, warnings, sys, inspect, imp
 import random, doctest, time
 
-from twisted.python import reflect, log, failure, modules
+from twisted.python import reflect, log, failure, modules, filepath
 from twisted.python.util import dsu
+from twisted.python.compat import set
+from twisted.python.lockfile import FilesystemLock
 
-from twisted.internet import defer, interfaces
+from twisted.internet import defer
 from twisted.trial import util, unittest
 from twisted.trial.itrial import ITestCase
+from twisted.trial.reporter import UncleanWarningsReporterWrapper
+
+# These are imported so that they remain in the public API for t.trial.runner
+from twisted.trial.unittest import suiteVisit, TestSuite
+
+from zope.interface import implements
 
 pyunit = __import__('unittest')
 
 
+
+class _WorkingDirectoryBusy(Exception):
+    """
+    A working directory was specified to the runner, but another test run is
+    currently using that directory.
+    """
+
+
+
+class _NoTrialMarker(Exception):
+    """
+    No trial marker file could be found.
+
+    Raised when trial attempts to remove a trial temporary working directory
+    that does not contain a marker file.
+    """
+
+
+
 def isPackage(module):
     """Given an object return True if the object looks like a package"""
     if not isinstance(module, types.ModuleType):
@@ -52,6 +76,7 @@ def samefile(filename1, filename2):
     """
     return os.path.abspath(filename1) == os.path.abspath(filename2)
 
+
 def filenameToModule(fn):
     """
     Given a filename, do whatever possible to return a module object matching
@@ -105,47 +130,21 @@ def _resolveDirectory(fn):
     return fn
 
 
-def suiteVisit(suite, visitor):
+
+class DestructiveTestSuite(TestSuite):
     """
-    Visit each test in C{suite} with C{visitor}.
-
-    @param visitor: A callable which takes a single argument, the L{TestCase}
-    instance to visit.
-    @return: None
+    A test suite which remove the tests once run, to minimize memory usage.
     """
-    for case in suite._tests:
-        visit = getattr(case, 'visit', None)
-        if visit is not None:
-            visit(visitor)
-        elif isinstance(case, pyunit.TestCase):
-            case = PyUnitTestCase(case)
-            case.visit(visitor)
-        elif isinstance(case, pyunit.TestSuite):
-            suiteVisit(case, visitor)
-        else:
-            case.visit(visitor)
-
-
-class TestSuite(pyunit.TestSuite):
-    """
-    Extend the standard library's C{TestSuite} with support for the visitor
-    pattern and a consistently overrideable C{run} method.
-    """
-
-    visit = suiteVisit
-
-    def __call__(self, result):
-        return self.run(result)
 
     def run(self, result):
         """
-        Call C{run} on every member of the suite.
+        Almost the same as L{TestSuite.run}, but with C{self._tests} being
+        empty at the end.
         """
-        # we implement this because Python 2.3 unittest defines this code
-        # in __call__, whereas 2.4 defines the code in run.
-        for test in self._tests:
+        while self._tests:
             if result.shouldStop:
                 break
+            test = self._tests.pop(0)
             test(result)
         return result
 
@@ -183,20 +182,27 @@ class LoggedSuite(TestSuite):
 
 class DocTestSuite(TestSuite):
     """
+    DEPRECATED in Twisted 8.0.
+
     Behaves like doctest.DocTestSuite, but decorates individual TestCases so
     they support visit and so that id() behaviour is meaningful and consistent
     between Python versions.
     """
 
     def __init__(self, testModule):
+        warnings.warn("DocTestSuite is deprecated in Twisted 8.0.",
+                      category=DeprecationWarning, stacklevel=2)
         TestSuite.__init__(self)
         suite = doctest.DocTestSuite(testModule)
         for test in suite._tests: #yay encapsulation
-            self.addTest(DocTestCase(test))
+            self.addTest(ITestCase(test))
+
 
 
 class PyUnitTestCase(object):
     """
+    DEPRECATED in Twisted 8.0.
+
     This class decorates the pyunit.TestCase class, mainly to work around the
     differences between unittest in Python 2.3, 2.4, and 2.5. These
     differences are::
@@ -210,6 +216,8 @@ class PyUnitTestCase(object):
     """
 
     def __init__(self, test):
+        warnings.warn("Deprecated in Twisted 8.0.",
+                      category=DeprecationWarning)
         self._test = test
         test.id = self.id
 
@@ -233,7 +241,11 @@ class PyUnitTestCase(object):
         """
         Call the given visitor with the original, standard library, test case
         that C{self} wraps. See L{unittest.TestCase.visit}.
+
+        Deprecated in Twisted 8.0.
         """
+        warnings.warn("Test visitors deprecated in Twisted 8.0",
+                      category=DeprecationWarning)
         visitor(self._test)
 
 
@@ -241,7 +253,12 @@ class PyUnitTestCase(object):
         return getattr(self._test, name)
 
 
+
 class DocTestCase(PyUnitTestCase):
+    """
+    DEPRECATED in Twisted 8.0.
+    """
+
     def id(self):
         """
         In Python 2.4, doctests have correct id() behaviour. In Python 2.3,
@@ -271,9 +288,6 @@ class TrialSuite(TestSuite):
         reactor.addSystemEventTrigger('after', 'shutdown',
                                       lambda: d.callback(None))
         reactor.fireSystemEvent('shutdown') # radix's suggestion
-        treactor = interfaces.IReactorThreads(reactor, None)
-        if treactor is not None:
-            treactor.suggestThreadPoolSize(0)
         # As long as TestCase does crap stuff with the reactor we need to
         # manually shutdown the reactor here, and that requires util.wait
         # :(
@@ -312,14 +326,9 @@ def isTestCase(obj):
     otherwise. Used to find all the tests in a module.
     """
     try:
-        return ITestCase.implementedBy(obj)
+        return issubclass(obj, pyunit.TestCase)
     except TypeError:
         return False
-    except AttributeError:
-        # Working around a bug in zope.interface 3.1.0; this isn't the user's
-        # fault, so we won't emit a warning.
-        # See http://www.zope.org/Collectors/Zope3-dev/470.
-        return False
 
 
 
@@ -329,6 +338,8 @@ class TestHolder(object):
     is concerned, this looks exactly like a unit test.
     """
 
+    implements(ITestCase)
+
     def __init__(self, description):
         """
         @param description: A string to be displayed L{TestResult}.
@@ -418,7 +429,6 @@ class TestLoader(object):
         self.suiteFactory = TestSuite
         self.sorter = name
         self._importErrors = []
-        self.forceGarbageCollection = False
 
     def sort(self, xs):
         """
@@ -514,9 +524,7 @@ class TestLoader(object):
         return self._makeCase(method.im_class, method.__name__)
 
     def _makeCase(self, klass, methodName):
-        test = klass(methodName)
-        test.forceGarbageCollection = self.forceGarbageCollection
-        return test
+        return klass(methodName)
 
     def loadPackage(self, package, recurse=False):
         """
@@ -574,7 +582,19 @@ class TestLoader(object):
         if not inspect.ismodule(module):
             warnings.warn("trial only supports doctesting modules")
             return
-        return DocTestSuite(module)
+        extraArgs = {}
+        if sys.version_info > (2, 4):
+            # Work around Python issue2604: DocTestCase.tearDown clobbers globs
+            def saveGlobals(test):
+                """
+                Save C{test.globs} and replace it with a copy so that if
+                necessary, the original will be available for the next test
+                run.
+                """
+                test._savedGlobals = getattr(test, '_savedGlobals', test.globs)
+                test.globs = test._savedGlobals.copy()
+            extraArgs['setUp'] = saveGlobals
+        return doctest.DocTestSuite(module, **extraArgs)
 
     def loadAnything(self, thing, recurse=False):
         """
@@ -632,7 +652,7 @@ class TestLoader(object):
             except:
                 errors.append(ErrorHolder(name, failure.Failure()))
         suites = [self.loadAnything(thing, recurse)
-                  for thing in sets.Set(things)]
+                  for thing in set(things)]
         suites.extend(errors)
         return self.suiteFactory(suites)
 
@@ -686,30 +706,77 @@ class TrialRunner(object):
                     dbg.rcLines.extend(rcFile.readlines())
         return dbg
 
+
+    def _removeSafely(self, path):
+        """
+        Safely remove a path, recursively.
+
+        If C{path} does not contain a node named C{_trial_marker}, a
+        L{_NoTrialmarker} exception is raised and the path is not removed.
+        """
+        if not path.child('_trial_marker').exists():
+            raise _NoTrialMarker(
+                '%r is not a trial temporary path, refusing to remove it'
+                % (path,))
+
+        try:
+            path.remove()
+        except OSError, e:
+            print ("could not remove %r, caught OSError [Errno %s]: %s"
+                   % (path, e.errno, e.strerror))
+            try:
+                newPath = FilePath('_trial_temp_old%s'
+                                   % random.randint(0, 99999999))
+                path.moveTo(newPath)
+            except OSError, e:
+                print ("could not rename path, caught OSError [Errno %s]: %s"
+                       % (e.errno,e.strerror))
+                raise
+
+
     def _setUpTestdir(self):
         self._tearDownLogFile()
         currentDir = os.getcwd()
-        testdir = os.path.normpath(os.path.abspath(self.workingDirectory))
-        if os.path.exists(testdir):
-           try:
-               shutil.rmtree(testdir)
-           except OSError, e:
-               print ("could not remove %r, caught OSError [Errno %s]: %s"
-                      % (testdir, e.errno,e.strerror))
-               try:
-                   os.rename(testdir,
-                             os.path.abspath("_trial_temp_old%s"
-                                             % random.randint(0, 99999999)))
-               except OSError, e:
-                   print ("could not rename path, caught OSError [Errno %s]: %s"
-                          % (e.errno,e.strerror))
-                   raise
-        os.mkdir(testdir)
-        os.chdir(testdir)
+        base = filepath.FilePath(self.workingDirectory)
+        counter = 0
+        while True:
+            if counter:
+                testdir = base.sibling('%s-%d' % (base.basename(), counter))
+            else:
+                testdir = base
+
+            self._testDirLock = FilesystemLock(testdir.path + '.lock')
+            if self._testDirLock.lock():
+                # It is not in use
+                if testdir.exists():
+                    # It exists though - delete it
+                    self._removeSafely(testdir)
+                break
+            else:
+                # It is in use
+                if self.workingDirectory == '_trial_temp':
+                    counter += 1
+                else:
+                    raise _WorkingDirectoryBusy()
+
+        testdir.makedirs()
+        os.chdir(testdir.path)
+        file('_trial_marker', 'w').close()
         return currentDir
 
+
+    def _tearDownTestdir(self, oldDir):
+        os.chdir(oldDir)
+        self._testDirLock.unlock()
+
+
+    _log = log
     def _makeResult(self):
-        return self.reporterFactory(self.stream, self.tbformat, self.rterrors)
+        reporter = self.reporterFactory(self.stream, self.tbformat,
+                                        self.rterrors, self._log)
+        if self.uncleanWarnings:
+            reporter = UncleanWarningsReporterWrapper(reporter)
+        return reporter
 
     def __init__(self, reporterFactory,
                  mode=None,
@@ -718,25 +785,24 @@ class TrialRunner(object):
                  profile=False,
                  tracebackFormat='default',
                  realTimeErrors=False,
-                 workingDirectory=None):
+                 uncleanWarnings=False,
+                 workingDirectory=None,
+                 forceGarbageCollection=False):
         self.reporterFactory = reporterFactory
         self.logfile = logfile
         self.mode = mode
         self.stream = stream
         self.tbformat = tracebackFormat
         self.rterrors = realTimeErrors
+        self.uncleanWarnings = uncleanWarnings
         self._result = None
         self.workingDirectory = workingDirectory or '_trial_temp'
         self._logFileObserver = None
         self._logFileObject = None
-        self._logWarnings = False
+        self._forceGarbageCollection = forceGarbageCollection
         if profile:
             self.run = util.profiled(self.run, 'profile.data')
 
-    def _setUpLogging(self):
-        self._setUpLogFile()
-        self._setUpLogWarnings()
-
     def _tearDownLogFile(self):
         if self._logFileObserver is not None:
             log.removeObserver(self._logFileObserver.emit)
@@ -755,54 +821,67 @@ class TrialRunner(object):
         self._logFileObserver = log.FileLogObserver(logFile)
         log.startLoggingWithObserver(self._logFileObserver.emit, 0)
 
-    def _setUpLogWarnings(self):
-        if self._logWarnings:
-            return
-        def seeWarnings(x):
-           if x.has_key('warning'):
-               print
-               print x['format'] % x
-        log.addObserver(seeWarnings)
-        self._logWarnings = True
 
     def run(self, test):
         """
         Run the test or suite and return a result object.
         """
+        test = unittest.decorate(test, ITestCase)
+        if self._forceGarbageCollection:
+            test = unittest.decorate(
+                test, unittest._ForceGarbageCollectionDecorator)
+        return self._runWithoutDecoration(test)
+
+
+    def _runWithoutDecoration(self, test):
+        """
+        Private helper that runs the given test but doesn't decorate it.
+        """
         result = self._makeResult()
         # decorate the suite with reactor cleanup and log starting
         # This should move out of the runner and be presumed to be
         # present
         suite = TrialSuite([test])
         startTime = time.time()
-        result.write("Running %d tests.\n", suite.countTestCases())
         if self.mode == self.DRY_RUN:
-            suite.visit(DryRunVisitor(result).markSuccessful)
-        elif self.mode == self.DEBUG:
-            # open question - should this be self.debug() instead.
-            debugger = self._getDebugger()
-            oldDir = self._setUpTestdir()
-            try:
-                self._setUpLogging()
-                debugger.runcall(suite.run, result)
-            finally:
-                os.chdir(oldDir)
+            for single in unittest._iterateTests(suite):
+                result.startTest(single)
+                result.addSuccess(single)
+                result.stopTest(single)
         else:
+            if self.mode == self.DEBUG:
+                # open question - should this be self.debug() instead.
+                debugger = self._getDebugger()
+                run = lambda: debugger.runcall(suite.run, result)
+            else:
+                run = lambda: suite.run(result)
+
             oldDir = self._setUpTestdir()
             try:
-                self._setUpLogging()
-                suite.run(result)
+                self._setUpLogFile()
+                run()
             finally:
-                os.chdir(oldDir)
+                self._tearDownLogFile()
+                self._tearDownTestdir(oldDir)
+
         endTime = time.time()
-        result.printErrors()
-        result.writeln(result.separator)
-        result.writeln('Ran %d tests in %.3fs', result.testsRun,
-                       endTime - startTime)
-        result.write('\n')
-        result.printSummary()
+        done = getattr(result, 'done', None)
+        if done is None:
+            warnings.warn(
+                "%s should implement done() but doesn't. Falling back to "
+                "printErrors() and friends." % reflect.qual(result.__class__),
+                category=DeprecationWarning, stacklevel=3)
+            result.printErrors()
+            result.writeln(result.separator)
+            result.writeln('Ran %d tests in %.3fs', result.testsRun,
+                           endTime - startTime)
+            result.write('\n')
+            result.printSummary()
+        else:
+            result.done()
         return result
 
+
     def runUntilFailure(self, test):
         """
         Repeatedly run C{test} until it fails.
@@ -811,9 +890,13 @@ class TrialRunner(object):
         while True:
             count += 1
             self.stream.write("Test Pass %d\n" % (count,))
-            result = self.run(test)
+            if count == 1:
+                result = self.run(test)
+            else:
+                result = self._runWithoutDecoration(test)
             if result.testsRun == 0:
                 break
             if not result.wasSuccessful():
                 break
         return result
+
